
local L_RARE_REMAINING = "|TInterface\\AddOns\\BetterClassHallMissions\\Textures\\Mobile-Clock:0|t %s"

local mechanicIcons = {
	[1380306] = [[Interface\AddOns\BetterClassHallMissions\Textures\MechanicIcon-Curse-Shadow]],
	[1383683] = [[Interface\AddOns\BetterClassHallMissions\Textures\MechanicIcon-Disorienting-Shadow]],
	[1383682] = [[Interface\AddOns\BetterClassHallMissions\Textures\MechanicIcon-Lethal-Shadow]],
	[1390116] = [[Interface\AddOns\BetterClassHallMissions\Textures\MechanicIcon-Powerful-Shadow]],
	[1383681] = [[Interface\AddOns\BetterClassHallMissions\Textures\MechanicIcon-Slowing-Shadow]],
}

local function FormatTime(time)
	if time >= 86400 then
		time = (time - 1) / 86400 + 1
		return COOLDOWN_DURATION_DAYS:format(time)
	elseif time >= 3600 then
		time = (time - 1) / 3600 + 1
		return COOLDOWN_DURATION_HOURS:format(time)
	else
		time = (time - 1) / 60 + 1
		return COOLDOWN_DURATION_MIN:format(time)
	end
end

local function CreateThreatFrame(parent)
	local threat = CreateFrame("Frame", nil, parent, "GarrisonAbilityCounterWithCheckTemplate")
	threat.Border:Hide()
	threat.Away:Hide()
	threat.Check:SetAtlas("GarrMission_CounterCheck", true)
	threat.TimeLeft:SetFontObject(SystemFont_Shadow_Med1_Outline) -- SystemFont_Outline_Small
	threat.TimeLeft:ClearAllPoints()
	threat.TimeLeft:SetPoint("CENTER", threat.Away, "CENTER", 0, 0)
	return threat
end

local function ShowThreats(threatsFrame, missionID)
	-- this creates a fair amount of garbage
	local _, _, _, _, _, _, _, enemies = C_Garrison.GetMissionInfo(missionID)
	local counterableThreats = _G.GarrisonMission_DetermineCounterableThreats(missionID, LE_FOLLOWER_TYPE_GARRISON_7_0)

	local numThreats = 0
	for _, enemy in ipairs(enemies) do
		for mechanicID, mechanic in next, enemy.mechanics do
			numThreats = numThreats + 1
			local threat = threatsFrame[numThreats]
			if not threat then
				threat = CreateThreatFrame(threatsFrame)
				threat:SetPoint("LEFT", threatsFrame[numThreats - 1], "RIGHT", 10, 0)
				tinsert(threatsFrame, threat)
			end

			threat.Icon:SetTexture(mechanicIcons[mechanic.ability.icon] or mechanic.ability.icon)
			threat.Check:Hide()
			threat.TimeLeft:Hide()
			threat.Working:Hide()

			if counterableThreats.full[mechanicID] and counterableThreats.full[mechanicID] > 0 then
				counterableThreats.full[mechanicID] = counterableThreats.full[mechanicID] - 1
				threat.Check:Show()
			elseif counterableThreats.away[mechanicID] and #counterableThreats.away[mechanicID] > 0 then
				local soonestTime = tremove(counterableThreats.away[mechanicID], 1)
				threat.TimeLeft:SetText(FormatTime(soonestTime))
				threat.TimeLeft:Show()
			elseif counterableThreats.worker[mechanicID] and counterableThreats.worker[mechanicID] > 0 then
				counterableThreats.worker[mechanicID] = counterableThreats.worker[mechanicID] - 1
				threat.Working:Show()
			end

			threat:Show()
		end
	end
	for i = numThreats + 1, #threatsFrame do
		threatsFrame[i]:Hide()
	end
	threatsFrame:SetWidth(30 * numThreats)
	threatsFrame:Show()
end

local SetRewards
do
	local cache = {}
	local tooltip = CreateFrame("GameTooltip", "BetterClassHallMissionsTooltip", nil, "GameTooltipTemplate")
	tooltip:SetOwner(WorldFrame, "ANCHOR_NONE")

	local function GetArtifactPower(itemID)
		if cache[itemID] == nil then
			-- cache[itemID] = false
			tooltip:SetItemByID(itemID)
			local text = _G["BetterClassHallMissionsTooltipTextLeft2"]:GetText()
			if text and text:match(_G.ARTIFACT_POWER) then
				text = _G["BetterClassHallMissionsTooltipTextLeft4"]:GetText()
				local ap = text:match("(%d?%d?,?%d+)")
				if ap then
					cache[itemID] = ap
				end
			end
		end
		return cache[itemID]
	end

	function SetRewards(self)
		for _, reward in ipairs(self.Rewards) do
			if reward.itemID then
				local ap = GetArtifactPower(reward.itemID)
				if ap then
					reward.Quantity:SetFormattedText("|cffe5cc7f%s|r", ap)
					reward.Quantity:Show()
				end
			end
		end
	end
end

hooksecurefunc(OrderHallMissionFrame.MissionTab.MissionList, "Update", function(self)
	if not OrderHallMissionFrame:IsShown() or not OrderHallMissionFrame.abilityCountersForMechanicTypes then return end

	for _, button in ipairs(self.listScroll.buttons) do
		local threatsFrame = button.threatsFrame
		if threatsFrame then threatsFrame:Hide() end

		local mission = button.info
		if mission then
			if not mission.inProgress then
				button.Title:ClearAllPoints()
				button.Title:SetPoint("LEFT", 160, 10)
				if button.Title:GetWidth() + button.Summary:GetWidth() + 8 > 545 then
					button.Title:SetWidth(545)
				end
				button.Summary:ClearAllPoints()
				button.Summary:SetPoint("BOTTOMLEFT", button.Title, "BOTTOMRIGHT", 8, 0)

				if not threatsFrame then
					threatsFrame = CreateFrame("Frame", nil, button)
					threatsFrame:SetPoint("TOPLEFT", button.Title, "BOTTOMLEFT", 0, 0)
					threatsFrame:SetSize(0, 28)

					local threat = CreateThreatFrame(threatsFrame)
					threat:SetPoint("TOPLEFT", 0, -4)
					threatsFrame[1] = threat

					local timeRemaining = threatsFrame:CreateFontString(nil, "OVERLAY")
					timeRemaining:SetPoint("LEFT", threatsFrame, "RIGHT", 20, 0)
					timeRemaining:SetFontObject(GameFontHighlightLarge)
					timeRemaining:SetTextColor(0.8, 0.8, 0.8)
					timeRemaining:SetText("")
					threatsFrame.timeRemaining = timeRemaining

					button.threatsFrame = threatsFrame
				end
				ShowThreats(threatsFrame, mission.missionID)

				if not mission.isRare then
					threatsFrame.timeRemaining:SetText("")
				else
					threatsFrame.timeRemaining:SetFormattedText(L_RARE_REMAINING, mission.offerTimeRemaining)
				end
			end

			-- put the ap value on the reward button
			SetRewards(button)
		end
	end
end)
